# ---------------------------------------------------------------------------
# Copyright 2002 Benjamin Trott
# Copyright 2003 Dwight Shih
#
# Functions for handling and displaying comments
# Derived from the Standalone Trackback package released by Benjamin Trott.
#
# This code is released under the Artistic License.
#
# $Id: service.pm,v 1.9 2004/10/05 10:58:44 dwight Exp $
# ---------------------------------------------------------------------------

package comment::service;

use strict;
use bradchoate::sanitize qw( sanitize_html );
use vars qw($VERSION @ISA);

require POSIX;

use Exporter;
use CGI qw( :cgi );
use File::Spec::Functions;

use trackback::core;

@ISA = qw(Exporter trackback::core);
$VERSION = '1.00';

# ---------------------------------------------------------------------------

sub new {
    my $class = shift;
    my %param = @_;
    my $self  = trackback::core->new(@_);
    bless($self,$class);

    #
    if ( $param{ALWAYS_APPROVED} ) {
        $self->{ALWAYS_APPROVED} = $param{ALWAYS_APPROVED};
    }
    else {
        $self->{ALWAYS_APPROVED} = 0;
    }
    #
    if ( $param{HEADER} ) {
        $self->{HEADER} = $param{HEADER};
    } else {
        $self->{HEADER} = <<HEADER_END;
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">

<html>
<head>
<title>Comments</title>
<base target="_blank" />
<meta name="robots" content="noindex,nofollow">
<style type="text/css">
body {
    color:black;
    background-color:#CCCCCC;
}
.even {
    color:black;
    background-color:white;
    padding-top: 5px;
    padding-bottom: 5px;
}
.odd {
    color:black;
    background-color:#CCCCCC;
    padding-top: 5px;
    padding-bottom: 5px;
}
</style>
</head>
<body>
HEADER_END
    }

    #
    if ( $param{BODY_FUNCTION} ) {
        $self->{BODY_FUNCTION} = $param{BODY_FUNCTION};
    } else {
        $self->{BODY_FUNCTION} = \&_comment_display;
    }

    #
    if ( $param{FORM} ) {
        $self->{FORM} = $param{FORM};
    } else {
        $self->{FORM} = <<FORM_END;
<form method="POST" target="_self">
<input type="hidden" name="%s" value="%s">
<input type="hidden" name="link" value="%s">
<input type="hidden" name="%s" value="ping">
<table>
<tr><td colspan="2"><b>Add your comment</b><br />(a, b, i, p tags allowed, tags subject to moderation):</td></tr>
<tr><td valign="top" width="1%">Name:</td>
    <td valign="top" width="99%"><input type="text" name="blog_name" size="50" maxlength="255" /></td>
</tr>
<tr><td valign="top">Web site:</td>
    <td valign="top"><input type="text" name="url" size="50" maxlength="255" /></td>
</tr>
<tr><td valign="top" width="1%">Comment:</td>
    <td valign="top"><textarea name="excerpt" rows="7" cols="80" style="width:100%"></textarea></td>
</tr>
<tr><td>&nbsp;</td>
    <td><input type="submit" name="submitComment" value="&nbsp;Submit&nbsp;" />&nbsp;&nbsp;<input type="button" name="cancelComment" value="&nbsp;Cancel&nbsp;" onclick="window.close()" /></td>
</table>
</form>
FORM_END
    }

    #
    if ( $param{FOOTER} ) {
        $self->{FOOTER} = $param{FOOTER};
    } else {
        $self->{FOOTER} = <<FOOTER_END;
</body>
</html>
FOOTER_END
    }

    #
    if ( $param{RSS_FILE} ) {
        $self->{RSS_FILE} = $param{RSS_FILE};
    } else {
        $self->{RSS_FILE} = undef;
    }
    
    return $self;
}

#
#
sub handle {
    my $self = shift;

    my $mode = $self->get_mode();
    my $id   = $self->get_id();
    my $link = param("link");

    if ( $mode ) {
        if ( $mode eq "counts" ) {
            $self->comment_counts();
        } elsif ( $mode eq "ping" ) {
            $self->comment_ping( $id,$link );
        } elsif ( $mode eq "list" && $id ) {
            $self->comment_listing( $id );
        } else {
            $self->exit_with_error( "Bad Command Request" );
        }
    } elsif ( $id ) {
        $self->comment_listing( $id );
    } else {
        $self->exit_with_error( "Bad Request" );
    }
}

#
#
sub comment_counts {
    my $self = shift;
    print <<COUNT_END;
Content-Type: text/html

function commentCounter( id ) {
    if (cmCount["t_"+id] ) {
        document.write(cmCount["t_"+id]);
    } else {
        document.write(\"0\");
    }
}
COUNT_END
    print "var cmCount = { ";
    my $count_data = $self->count_load();
    my ($key,$value);
    while ( ($key,$value) = each(%$count_data) ) {
        print "t_$key: $value, ";
    }
    print "tb_null: 0 };\n";
}

#
#
sub comment_ping {
    my ($self, $id, $link) = @_;

    $self->exit_with_error( sprintf( "No comment id (%s)",$self->{ID_PARAMETER}) ) 
        unless $id;

    if ( request_method =~ /POST/ ) {
        my $ping = { map { $_ => scalar param($_) } qw(excerpt url blog_name) };

        if ( $ping->{excerpt} ) {

            $ping->{timestamp} = time;
            $ping->{ipaddr}    = $ENV{REMOTE_ADDR};
            $ping->{approved}  = $self->{ALWAYS_APPROVED};

            if ( $self->has_banned_link( $ping->{excerpt} ) ) {
                $self->placebo_display( $id,$ping );
                return;
            }

            $self->ping_store( $id, $ping );
            
            if ( $self->{RSS_FILE} && open(FH, ">" . $self->{RSS_FILE}) ) {
                print FH $self->rss_summary( "Comment" );
                close FH;
            }
        }
    }

    print redirect(url() . "?$self->{ID_PARAMETER}=$id&link=$link");
}

#
#
sub comment_listing {
    my $self     = shift;
    my $id       = shift;
    my $data     = $self->data_load($id);
    my $link     = param("link");

    print "Content-type: text/html\n\n";
    print $self->{HEADER};
    if ( $link ) {
        print qq(Listed below are comments to <a href="$link">this post</a>:<p>);
    }

    $self->{BODY_FUNCTION}($data);
    printf $self->{FORM}, $self->{ID_PARAMETER}, $id, $link, $self->{MODE_PARAMETER};
    print $self->{FOOTER};
}

sub _comment_display {
    my ($data) = @_;

    my $tmpl = <<TEMPLATE;
<div class="%s">
<div class="comment">%s</div>
<div class="nameDate">%s &#0149; %s</div>
</div>
TEMPLATE
    
    my $i = 0;
    for my $item (@$data) {
        my $ts = POSIX::strftime("%d %b %Y, %I:%M %p %Z", localtime($item->{timestamp}) );
        my $name = $item->{blog_name} || "Anonymous Coward";

        if ( $item->{approved} ) {
            my $nameUrl = $item->{url}
                        ? qq(<a href="$item->{url}" target="_new">$name</a>)
                        : $name;
            printf $tmpl, ($i % 2 ? "odd" : "even"), $item->{excerpt}, $nameUrl, $ts;
        } else {
            printf $tmpl, ($i % 2 ? "odd" : "even"), sanitize_html($item->{excerpt},"p"), $name, $ts;
        }

        $i++;
    }
    print "<hr>\n";
}

#
#
sub placebo_display {
    my ($self,$id,$ping) = @_;

    my $link     = param("link");
    my $name     = $ping->{blog_name} || "Anonymous Coward";
    my $ts       = POSIX::strftime("%d %b %Y, %I:%M %p %Z", localtime($ping->{timestamp}) );

    print "Content-type: text/html\n\n";
    print $self->{HEADER};
    if ( $link ) {
        print qq(Listed below are comments to <a href="$link">this post</a>:<p>);
    }

    my $tmpl = <<TEMPLATE;
<div class="even">
<div class="comment">Comment rejected (contains banned link):<br />%s</div>
<div class="nameDate">%s &#0149; %s</div>
</div>
TEMPLATE

    printf $tmpl, sanitize_html($ping->{excerpt},"p"), $name, $ts;

    printf $self->{FORM}, $self->{ID_PARAMETER}, $id, $link, $self->{MODE_PARAMETER};
    print $self->{FOOTER};
}

1;
